/*
aspilib.h -- ASPI Library Header File

Copyright (C) 1993 Roger C. Pao <rpao@mts.mivj.ca.us>
*/


/* Define FAR if required */
#define FAR /* __far */

/* Define BYTE and DWORD if needed before including this header file. */

#if !defined(BYTE)
#define BYTE unsigned char
#endif

#if !defined(DWORD)
#define DWORD unsigned long
#endif


#define MAX_CDB_LENGTH 12
#define MAX_SENSE_LENGTH 18


/* Valid ASPI Command Codes */
#define ASPI_CMD_HA_INQUIRY			0x00
#define ASPI_CMD_GET_DEVICE_TYPE		0x01
#define ASPI_CMD_EXEC_IO			0x02
#define ASPI_CMD_ABORT_IO			0x03
#define ASPI_CMD_RESET_SCSI_DEVICE		0x04
#define ASPI_CMD_SET_HA_PARAMS			0x05
#define ASPI_CMD_GET_DISK_DRIVE_INFO		0x06

/* ASPI (Request) Status */
#define ASPI_STATUS_REQ_IN_PROGRESS		0x00
#define ASPI_STATUS_REQ_COMPLETED_WITHOUT_ERR	0x01
#define ASPI_STATUS_REQ_ABORTED_BY_HOST		0x02
#define ASPI_STATUS_REQ_COMPLETED_WITH_ERROR	0x04
#define ASPI_STATUS_INVALID_SCSI_REQUEST	0x80
#define ASPI_STATUS_INVALID_HA_NUMBER		0x81
#define ASPI_STATUS_DEVICE_NOT_INSTALLED	0x82

/* Host Adapter Status */
#define ASPI_STATUS_HA_NO_ERROR			0x00
#define ASPI_STATUS_HA_SELECTION_TIMEOUT	0x11
#define ASPI_STATUS_HA_DATA_OVERRUN_UNDERRUN	0x12
#define ASPI_STATUS_HA_UNEXPECTED_BUS_FREE	0x13
#define ASPI_STATUS_HA_PHASE_SEQUENCE_ERROR	0x14

/* Target Status */
#define ASPI_STATUS_TARGET_NO_STATUS		0x00
#define ASPI_STATUS_TARGET_CHECK_STATUS		0x02
#define ASPI_STATUS_TARGET_BUSY			0x08
#define ASPI_STATUS_TARGET_RESERVATION_CONFLICT	0x18

/* SCSI Request Flags */
#define ASPI_REQ_FLAG_POST			0x01
#define ASPI_REQ_FLAG_LINK			0x02
#define ASPI_REQ_FLAG_DIR_MASK			0x18
#define ASPI_REQ_FLAG_DIR_ANY			0x00
#define ASPI_REQ_FLAG_DIR_TARGET_TO_HOST	0x08
#define ASPI_REQ_FLAG_DIR_HOST_TO_TARGET	0x10
#define ASPI_REQ_FLAG_DIR_NO_DATA_XFER		0x18

/* Drive Flags */
#define ASPI_DRV_FLAG_INT13_MASK		0x03
#define ASPI_DRV_FLAG_INT13_DOS_CONTROLLED	0x01
#define ASPI_DRV_FLAG_INT13_NOT_DOS_CONTROLLED	0x02
#define ASPI_DRV_FLAG_INT13_INVALID		0x03


#if !defined(_ASPILIB_TYPEDEF)
#define _ASPILIB_TYPEDEF

#pragma pack(1)

typedef struct _SRB;
typedef struct _SRB __far *PSRB;

typedef struct _SRB_HDR
{
				/* Ofs Byts */
	BYTE CommandCode;	/* 00h 01h SRB Command code  */
	BYTE Status;		/* 01h 01h ASPI (Request) Status */
	BYTE HAN;		/* 02h 01h Host Adapter Number */
	BYTE ScsiReqFlags;	/* 03h 01h SCSI Request Flags */
	BYTE Reserved[4];	/* 04h 04h Reserved for Expansion = 0 */
} SRB_HDR;

typedef struct _SRB_HAI
{
				/* Ofs Byts */
	/* SRB_HDR Header;	/* 00h 08h */
	BYTE HA_Count;		/* 08h 01h Number of Host Adapters */
	BYTE HA_ScsiId;		/* 09h 01h ID of Host Adapter */
	BYTE ScsiMgrId[16];	/* 0Ah 10h SCSI Manager ID */
	BYTE HA_Id[16];		/* 1Ah 10h Host Adapter ID */
	BYTE HA_Params[16];	/* 2Ah 10h Host Adapter unique params */
} SRB_HAI;

typedef struct _SRB_GetDevType
{
				/* Ofs Byts */
	/* SRB_HDR Header;	/* 00h 08h */
	BYTE TargetId;		/* 08h 01h SCSI ID of Target (0-7) */
	BYTE TargetLun;		/* 09h 01h SCSI LUN of Target (0-7) */
	BYTE PeriphDeviceType;	/* 0Ah 01h Peripheral Device Type of Target */
				/* It is assumed that this is byte 0 of
				   the Inquiry Command (12h)
				*/
} SRB_GetDevType;

typedef struct _SRB_ExecIo
{
				/* Ofs Byts */
	/* SRB_HDR Header;	/* 00h 08h */
	BYTE TargetId;		/* 08h 01h SCSI ID of Target (0-7) */
	BYTE TargetLun;		/* 09h 01h SCSI LUN of Target (0-7) */
	DWORD LengthData;	/* 0Ah 04h Data Allocation Length */
	BYTE LengthSense;	/* 0Eh 01h Sense Allocation Length */
	void FAR *DataPointer;	/* 0Fh 04h Data Buffer Pointer */
	PSRB LinkPointer;	/* 13h 04h SRB Link Pointer */
	BYTE LengthCDB;		/* 17h 01h SCSI CDB Length */
	BYTE HA_Status;		/* 18h 01h Host Adapter Status */
	BYTE TargetStatus;	/* 19h 01h Target SCSI Status */

	void (FAR *PostRoutinePtr)(PSRB SrbPointer);
				/* 1Ah 04h Post Routine Pointer */

	BYTE Reserved[34];	/* 1Eh 22h Reserved for ASPI Workspace */
	BYTE CDB[MAX_CDB_LENGTH];	/* 40h (Variable size)
				           SCSI Command Descriptor Block (CDB)
					*/
} SRB_ExecIo;

typedef struct _SRB_AbortIo
{
				/* Ofs Byts */
	/* SRB_HDR Header;	/* 00h 08h */
	PSRB SRB_Pointer;	/* 08h 04h SRB Pointer to Abort */
} SRB_AbortIo;

typedef struct _SRB_ResetDev
{
				/* Ofs Byts */
	/* SRB_HDR Header;	/* 00h 08h */
	BYTE TargetId;		/* 08h 01h SCSI ID of Target (0-7) */
	BYTE TargetLun;		/* 09h 01h SCSI LUN of Target (0-7) */
	BYTE Reserved[14];	/* 0Ah 01h Reserved = 0 */
	BYTE HA_Status;		/* 18h 01h Adapter status */
	BYTE TargetStatus;	/* 19h 01h Target SCSI status */

	void (FAR *PostRoutinePtr)(PSRB SrbPointer);
				/* 1Ah 04h Ptr to post routine */

	BYTE Reserved2[34];	/* 1Eh 01h Reserved */
} SRB_ResetDev;

typedef struct _SRB_SetHA_Params
{
				/* Ofs Byts */
	/* SRB_HDR Header;	/* 00h 08h */
	BYTE AdapterParams[16];	/* 08h 10h Host Adapter Unique Params */
} SRB_SetHA_Params;

typedef struct _SRB_GetDiskDrvInfo
{
				/* Ofs Byts */
	/* SRB_HDR Header;	/* 00h 08h */
	BYTE TargetId;		/* 08h 01h SCSI ID of Target (0-7) */
	BYTE TargetLun;		/* 09h 01h SCSI LUN of Target (0-7) */
	BYTE DriveFlags;	/* 0Ah 01h Drive Flags */
	BYTE Int13hDrive;	/* 0Bh 01h INT 13h Drive */
	BYTE HeadXlat;		/* 0Ch 01h Preferred Head Translation */
	BYTE SectorXlat;	/* 0Dh 01h Preferred Sector Translation */
	BYTE Reserved[10];	/* 0Eh 0Ah Reserved for Expansion = 0 */
} SRB_GetDiskDrvInfo;

typedef struct _SRB
{
	SRB_HDR Hdr;
	union _Cmd {
	  SRB_HAI HAI;
	  SRB_GetDevType GetDevType;
	  SRB_ExecIo ExecIo;
	  SRB_AbortIo AbortIo;
	  SRB_ResetDev ResetDev;
	  SRB_SetHA_Params SetHA_Params;
	  SRB_GetDiskDrvInfo GetDiskDrvInfo;
	} Cmd;
	BYTE SenseBuf[MAX_SENSE_LENGTH];
} SRB;

#pragma pack()


typedef void (__far *ASPI_ENTRY_TYPE)(PSRB Srb);


#endif


extern ASPI_ENTRY_TYPE AspiEntry;


int InitializeAspiEntry(void);
/* void AspiEntry(PSRB Srb); */
