/*
scsiping.c -- ScsiPing for DOS ASPI  V1.00

Copyright (C) 1993 Roger C. Pao <rpao@mts.mivj.ca.us>

10/23/93 rpao	Modified & posted to Usenet community.
09/18/93 rpao	Created: Sat Sep 18 10:19:37 PDT 1993
*/

#include <stdio.h>
#include <string.h>

typedef unsigned char uchar;
typedef unsigned long ulong;
#define BYTE uchar
#define DWORD ulong

#include "aspilib.h"


/* Many ASPI SCSI Managers crash miserably if HAN is not valid */
#define MIN_ADAPTER_NDX		0 /* -1 */
#define MAX_ADAPTER_NDX		AdapterCount /* AdapterCount+1 */

#define MAX_TARGET_ID		8
#define MAX_TARGET_LUN		8 /* Must use 0 for some SCSI targets */
#define MAX_INQUIRY_LENGTH	36
#define MAX_TMP			MAX_INQUIRY_LENGTH+1

BYTE TmpBuf[MAX_TMP];
SRB Srb;
int AdapterCount;


void SendSrb(PSRB Srb)
{
  AspiEntry(Srb);
  while (Srb->Hdr.Status==ASPI_STATUS_REQ_IN_PROGRESS)
    /* wait */;
}


void PrAdapterInquiry(int HAN)
{
  memset(&Srb,0,sizeof(SRB)); /* Clear Srb */
  Srb.Hdr.CommandCode=ASPI_CMD_HA_INQUIRY;
  Srb.Hdr.HAN=(BYTE)HAN;

  SendSrb(&Srb);

  Srb.Cmd.HAI.HA_Id[16]='\0';

  printf("Adapter %02X of %02X: <ID %d> '%-32s' [%02Xh]\n",
    HAN,AdapterCount,Srb.Cmd.HAI.HA_ScsiId,
    &Srb.Cmd.HAI.ScsiMgrId,
    (uchar)Srb.Hdr.Status);
}


void PrDeviceInquiry(int HAN, int TargetId, int TargetLun)
{
  /* Clear TmpBuf */
  memset(TmpBuf,0,8); memset(&TmpBuf[8],' ',MAX_INQUIRY_LENGTH-8);
  TmpBuf[MAX_INQUIRY_LENGTH]='\0';

  memset(&Srb,0,sizeof(SRB)); /* Clear Srb */
  Srb.Hdr.CommandCode=ASPI_CMD_EXEC_IO;
  Srb.Hdr.HAN=(BYTE)HAN;
  Srb.Hdr.ScsiReqFlags=ASPI_REQ_FLAG_DIR_ANY;
  Srb.Cmd.ExecIo.TargetId=(BYTE)TargetId;
  Srb.Cmd.ExecIo.TargetLun=(BYTE)TargetLun;
  Srb.Cmd.ExecIo.LengthData=36;
  Srb.Cmd.ExecIo.LengthSense=MAX_SENSE_LENGTH;
  Srb.Cmd.ExecIo.DataPointer=TmpBuf;
  Srb.Cmd.ExecIo.LengthCDB=6;
  Srb.Cmd.ExecIo.CDB[0]=0x12; /* Inquiry Command */
  Srb.Cmd.ExecIo.CDB[1]=(BYTE)(TargetLun<<5);
  Srb.Cmd.ExecIo.CDB[4]=36;

  SendSrb(&Srb);

  printf("(%02Xh) '%-28s' {%02Xh}[%02Xh][%02Xh][%02Xh]\n",
    (uchar)TmpBuf[0],&TmpBuf[8],
    (uchar)Srb.Cmd.ExecIo.CDB[Srb.Cmd.ExecIo.LengthCDB+2], /* SenseKey */
    (uchar)Srb.Hdr.Status,
    (uchar)Srb.Cmd.ExecIo.HA_Status,
    (uchar)Srb.Cmd.ExecIo.TargetStatus);
}


void main(void)
{
  int AdapterNdx, TargetId, TargetLun;

  if (InitializeAspiEntry())
    fprintf(stderr,"error: could not initialize ASPI entry\n");

  /* Host Adapter Inquiry: How many host adapters are out there? */
  memset(&Srb,0,sizeof(SRB)); /* Clear Srb */
  Srb.Hdr.CommandCode=ASPI_CMD_HA_INQUIRY;
  Srb.Hdr.HAN=0;
  SendSrb(&Srb);
  AdapterCount=Srb.Cmd.HAI.HA_Count;

  for (AdapterNdx=MIN_ADAPTER_NDX; AdapterNdx<MAX_ADAPTER_NDX; AdapterNdx++) {
    PrAdapterInquiry(AdapterNdx);
    for (TargetId=0; TargetId<MAX_TARGET_ID; TargetId++) {
      for (TargetLun=0; TargetLun<MAX_TARGET_LUN; TargetLun++) {

        printf("<ID %d LUN %d> ",TargetId,TargetLun);
	  PrDeviceInquiry(AdapterNdx,TargetId,TargetLun);

      } /* for AdapterNdx */
    } /* for TargetId */
  } /* for TargetLun */

}
